// Demo Program for MPI_Scatter and MPI_Gather
// Open source
// Peter-Bernd Otte, HI-Mainz

#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

int main (int argc, char *argv[]) {
if (argc != 3) {
    fprintf(stderr, "Usage: BINARYNAME n elements_per_rank\n");
    exit(1);
  }

  int n = atoi(argv[1]); //size of vector
  int elements_per_rank = atoi(argv[2]);

  int my_rank, size;

  double *a = NULL;

  double *local_a = (double *)malloc(sizeof(double) * elements_per_rank);

  MPI_Init(NULL, NULL);
  MPI_C...(MPI_COMM_WORLD, &my_rank);
  MPI_C...(MPI_COMM_WORLD, &size);

  if (my_rank == 0) {
    a = malloc(n*sizeof(double));
    printf("Enter vector values: \n");
    for (int i=0; i<n; i++) {
      scanf("%lf", &a[i]);
    }

    //Use MPI_Scatter to send the variable a (send_count: elements_per_rank) to the receiving clients
    //The receive the data into local_a (count: elements_per_rank)
    MPI_Scatter(...);
  } else {
    //Call MPI_Scatter witht he correct parameters
    MPI_Scatter(...);
  }

  // Print local result after MPI_Scatter
  printf("Values (rank %d): ", my_rank);
  ...
  printf("\n");


  // Do some computation
  // Compute the average of your subset
  double local_avg = 0;
  for (int i=0; i<elements_per_rank; i++) local_avg+=local_a[i];
  local_avg = local_avg/elements_per_rank;


  // Now collect all data from sub processes to rank 0
  double *res_a = NULL;
  if (my_rank == 0) {
    res_a = (double *)malloc(sizeof(double)*size);
  }
  //Use MPI_Gather to collect local_avg into res_a
  MPI_Gather(...);

  // Print result after MPI_Gather
  if (!my_rank) {
    printf("Values (rank %d): ", my_rank);
    ...
    printf("\n");
  }

  //Free up memeory
  if (!my_rank) {
    free(res_a);
    free(a);
  }
  free(local_a);

  MPI_Finalize();
}
